
#ifndef _HEXGRIDDISPLAY_H_
	#define _HEXGRIDDISPLAY_H_

#include <SDL2/SDL.h>
#include "RGBColor.h"
#include "XYLocation.h"
#include "HexDisplay.h"

class RealmColorMap;

class HexGridDisplay {

	private:
		int gridsize;
		int hexradius;

		// pixel position where the hex grid will be drawn
		XYLocation center;

		// hex position (in hex-draw units) of spot to put precisely at the center point defined above
		// (this will allow us to draw the grid centered on an arbitrary hex location,
		// and any position offset due to rounding errors will be as far from focus as possible)
		XYLocation centerhex;

		bool FillEnabled;

		RGBColor LineColor;
		RGBColor FillColor;

		bool inverted;



		RealmColorMap* BackgroundColors;


	public:
		HexGridDisplay(bool invert);

		// setCenter : the pixel location to place the hex grid
		void setCenter(XYLocation c) { center = c; }
		void setLineColor(RGBColor lc) {
			LineColor = lc;
		}

		void setFillColor(RGBColor fc) {
			FillColor = fc;
		}

		void setGridSize(int gsize) { gridsize = gsize; }

		void attachColorMap(RealmColorMap* tp) { BackgroundColors = tp; }

		void setHexRadius(int radius) {
			hexradius = radius;
		}

		void setFillEnabled(bool fe) {
			FillEnabled = fe;
		}

		// setCenterhex : the hex location (passed in hex-draw units) that will be centered
		void setCenterhex(XYLocation chex) { centerhex = chex; }

		// draw the grid to an SDL renderer
		void draw(SDL_Renderer* renderer);
};


#endif
