#ifndef __HEXSPIRALCOORDINATE_H
	#define __HEXSPIRALCOORDINATE_H

#include "XYLocation.h"
#include "const7.h"

/* information about a given hex coordinate in a spiral */
class HexSpiralCoordinate {
	private:

	long int HexIndex = 0;

	int rdistance = 0;  // current radial distance from center
	int phase = 0; // phase (side) of the hex the spot is in
	int poffset = 0; // how far from the beginning of the phase

	int circ = 1; // current circumference... increases by 6 with each round (initial value of 1 = single center hex)


	void calcLocationInfo();

	public:

	long int getHexIndex() const { return HexIndex; }
	void setHexIndex(long int i) {HexIndex = i; calcLocationInfo();}

	HexSpiralCoordinate(long int index);

	// derives spiral index from the constituent attributes
	HexSpiralCoordinate(int rdistance, int phase, int poffset);

	private:
	// this one is used just for optimized copying of self in movement
	// calculations and translations, not intended for external use,
	// instead public/outside code should call one of the public
	// constructors so remaining values can be derived
	HexSpiralCoordinate(long int index, int rdistance, int phase, int poffset, int circ);

	public:

	int getRadialDistance() { return rdistance; }
	int getSide() { return phase; }
	int getSidePosition() { return poffset; }

	HexSpiralCoordinate calcMovement(int direction);

	XYLocation getXY(bool is_inverted);

	HexSpiralCoordinate getTranslatedCoordinate(HexSpiralCoordinate offsetCoordinate);

	HexSpiralCoordinate operator + (const HexSpiralCoordinate& h) const;
	HexSpiralCoordinate operator - (const HexSpiralCoordinate& h) const;

	HexSpiralCoordinate getParentSpiralCoordinate(bool direction_forward = true, int map_radius = REALM_MAP_RADIUS);
	HexSpiralCoordinate getChildSpiralCoordinate(bool direction_forward = false, int map_radius = REALM_MAP_RADIUS);

	HexSpiralCoordinate findWrappedPosition(int radius, bool invert);
};

#endif // __HEXSPIRALCOORDINATE_H
