
#include "RealmSceneElevationMapRepository.h"
#include "RealmSceneElevationSubMap.h"
#include "RealmSceneElevationMasterMap.h"
#include "RealmMapDisplay.h"
#include "RealmSceneBackgroundRepository.h"


RealmSceneElevationMapRepository::~RealmSceneElevationMapRepository() {
	delete MasterMap;
	for (auto it = MapsByKey.begin(); it != MapsByKey.end(); ++it) {
		delete it->second;
	}
	for (auto it = MapsByScene.begin(); it != MapsByScene.end(); ++it) {
		delete it->second;
	}
}

void RealmSceneElevationMapRepository::init() {
	RealmSceneKey k(0, 0, 0, 0, 4);
	RealmMapKey mk(k, 0);
	MasterMap = new RealmSceneElevationMasterMap(this, mk);

	addMapByScene(k, 0, MasterMap);
}

RealmSceneElevationMap* RealmSceneElevationMapRepository::findSceneElevationMap(RealmMapKey k) {

	if (k.MapCenterKey.displaylevel >= 4) {
		return MasterMap;
	} else {
		return findSceneElevationSubMap(k);
	}
}


RealmSceneElevationSubMap* RealmSceneElevationMapRepository::findSceneElevationSubMap(RealmMapKey k)
{
	std::map<RealmMapKey,RealmSceneElevationSubMap *>::iterator it;
	it = MapsByKey.find(k);
	if (it != MapsByKey.end())
		return it->second;
	else
	{
		RealmSceneElevationSubMap* emptyone;

		RealmSceneElevationMap *parentmap;
		RealmSceneElevationSubMap *centermap;

		RealmMapKey pkey = k;
		pkey.MapCenterKey.displaylevel++;

		HexSpiralCoordinate parent_spiral_coord_offset =
			(
				HexSpiralCoordinate(k.MapCenterKey.getSpiralIndex()) +
				HexSpiralCoordinate(k.CenterSpiralOffset)
			).getParentSpiralCoordinate(!pkey.isInverted());

		pkey.CenterSpiralOffset = parent_spiral_coord_offset.getHexIndex();

		parentmap = findSceneElevationMap(pkey);

		// avoid corner case boundary issues in populating neighbor edge cases by pointing using the parent's center map
		RealmSceneElevationMap* parent_center_map = parentmap->getCenterMap();

		if (k.CenterSpiralOffset > 0) {
			RealmMapKey ckey = k;
			ckey.CenterSpiralOffset = 0;
			centermap = findSceneElevationSubMap(ckey);

			emptyone = new RealmSceneElevationSubMap(parent_center_map, centermap, k, this);

		} else {
			emptyone = new RealmSceneElevationSubMap(parent_center_map, nullptr, k, this);
			centermap = emptyone;
		}

		addSubMap(k, emptyone);
		return emptyone;
	}
}

void RealmSceneElevationMapRepository::addSubMap(RealmMapKey k, RealmSceneElevationSubMap* m) {
	MapsByKey.insert(std::pair<RealmMapKey, RealmSceneElevationSubMap*>(k, m));

	addMapByScene(k.MapCenterKey, k.CenterSpiralOffset, m);

	BackgroundRepository->makeSceneBackground(m);
}

void RealmSceneElevationMapRepository::addMapByScene(RealmSceneKey k, long int spiral_offset, RealmSceneElevationMap* m)
{
	std::map<long int, RealmSceneElevationMap*> *scene_maps;

	auto it = MapsByScene.find(k);
	if (it != MapsByScene.end()) {
		scene_maps = it->second;
	} else {
		scene_maps = new std::map<long int, RealmSceneElevationMap*>;
		MapsByScene.insert(std::pair<RealmSceneKey, std::map<long int, RealmSceneElevationMap*>*>(k, scene_maps));
	}

	scene_maps->insert(std::pair<long int, RealmSceneElevationMap*>(spiral_offset, m));
}

