#include <iostream>

#include "RealmTerrain.h"
#include "RealmSceneView.h"
#include "SceneMoveAnimator.h"

using namespace std;


RealmTerrain::RealmTerrain() :
	MapRepository(&BackgroundRepository)
{
	srand(time(0));
	foresttreecount = 79 * 3;
	scene1trees = new RealmSceneObject[foresttreecount];
}

void RealmTerrain::init(int base_radius) {

	MapRepository.init();

	SceneObjectCollection testscene;
	SceneObjectCollection testscene2;
	SceneObjectCollection testzone;
	SceneObjectCollection testregion;

	SceneObjectCollection forestscene;

	treetype.init("tree.svg");
	spiraltype.init("spiral1.svg");
	cursortype.init("star_cursor_spiraltop.svg");

	int curspace = 0;
	int inclevel = 1;
	int inccount = 0;
	int variance = 0;
	int spacediff = 0;
	HexSpiralCoordinate space_coord(curspace);
	HexSpiralCoordinate offset_coord(42);

	for (int treei = 0; treei < foresttreecount; treei++) {
		RealmLocation scenetreelocation;

		if (variance > 0) {
			if (inccount % 2 == 0) {
				spacediff = - (rand() % variance);
			} else {
				spacediff = (rand() % variance);
			}
		}



		scenetreelocation.setScene(1);
		//scenetreelocation.setSpace(curspace + spacediff);
		space_coord.setHexIndex(curspace + spacediff);

		HexSpiralCoordinate final_coord = space_coord.getTranslatedCoordinate(offset_coord);

		scenetreelocation.setSpaceCoordinate(final_coord);
		scene1trees[treei].init(&treetype, base_radius * (2 + inclevel / 5), base_radius * (2 + inclevel / 5), scenetreelocation, 1);
		forestscene.addObject(&scene1trees[treei]);

		curspace += inclevel;
		inccount++;
		if (inccount == 3) // (inccount == 6)
		{
			inclevel ++;
			inccount = 0;
			if (inclevel > 6) {
				// variance += 1;
			}
		}
	}

	XYLocation treelocation0(5, -6);
	XYLocation spirallocation0(0, 10);

	worldtree1.init(&treetype, base_radius * 5, base_radius * 5, treelocation0);
	worldspiral1.init(&spiraltype, base_radius * 4, base_radius * 4, spirallocation0);

	worldobjects.addObject(&worldtree1);
	worldobjects.addObject(&worldspiral1);

	RealmLocation centerlocation(0, 0, 0, 0, 0);

	RealmLocation treelocation(0, 0, 0, 3, 0);

	RealmLocation biglocation1(0, 0, 0, 0, 152);
	RealmLocation biglocation2(0, 0, 0, 0, 90);

	RealmLocation treelocation2(0, 0, 0, 10, 0);
	RealmLocation biglocation3(0, 0, 0, 10, 15);
	RealmLocation biglocation4(0, 0, 0, 10, 48);


	RealmLocation spirallocation(0, 0, 5, 0, 0);

	keycursor.init(&cursortype, base_radius, base_radius, centerlocation, 1);

	bigtree1.init(&treetype, base_radius * 6, base_radius * 6, biglocation1, 1 );
	bigtree2.init(&treetype, base_radius * 6, base_radius * 6, biglocation2, 1 );

	bigtree3.init(&treetype, base_radius * 6, base_radius * 6, biglocation3, 1);
	bigtree4.init(&treetype, base_radius * 7, base_radius * 7, biglocation4, 1);

	testtree.init(&treetype, base_radius * 2, base_radius * 2, treelocation, 2);
	testtree2.init(&treetype, base_radius * 3, base_radius * 3, treelocation2, 2);

	testspiral.init(&spiraltype, base_radius * 5, base_radius * 5, spirallocation, 3);

	testscene.addObject(&bigtree1);
	testscene.addObject(&bigtree2);
	testscene.addObject(&keycursor);

	testscene2.addObject(&bigtree3);
	testscene2.addObject(&bigtree4);

	testzone.addObject(&testtree);
	testzone.addObject(&testtree2);

	testregion.addObject(&testspiral);

	RealmSceneKey scene1key(biglocation1, 1);
	RealmSceneKey scene2key(biglocation3, 1);
	RealmSceneKey zonekey(treelocation, 2);
	RealmSceneKey regionkey(spirallocation, 3);

	RealmLocation forestlocation(0, 0, 0, 1, 0);
	RealmSceneKey forestkey(forestlocation, 1);

	scenerepo.addCollection(scene1key, testscene);
	scenerepo.addCollection(scene2key, testscene2);
	scenerepo.addCollection(zonekey, testzone);
	scenerepo.addCollection(regionkey, testregion);
	scenerepo.addCollection(forestkey, forestscene);
}

void RealmTerrain::autoMove() {
	SceneMoveAnimator* cursor_animator = keycursor.getAnimator();
	HexSpiralCoordinate target ( cursor_animator->getHexPosition().getHexIndex() + 1);
	cursor_animator->beginMove(target);
}

void RealmTerrain::cleanup() {

	delete[] scene1trees;

	treetype.cleanup();
	spiraltype.cleanup();
}


void RealmTerrain::updateObjectCollection(int view_level, RealmLocation view_center) {

	RealmSceneKey lockey(view_center, view_level);
	SceneObjects = scenerepo.findCollection(lockey);

	/*
	cout << "Retrieving objects at: "
		<< view_center.getRealm() << "," <<
		view_center.getRegion() << "," <<
		view_center.getZone() << "," <<
		view_center.getScene() << "," <<
		view_center.getSpace() << endl;

	cout << "Object count: " <<
		SceneObjects.getCount() << endl;
		*/

	if (view_level == 1) {
		XYLocation world_view_center = view_center.getRealmWorldCoordinate();

		std::vector<RealmWorldObject*> range_objects;

		// +-25 works fairly well for retrieving objects within the scene's default boundary
	/*
		worldobjects.getRange(
			world_view_center.x - 25, world_view_center.y - 25,
			world_view_center.x + 25, world_view_center.y + 25,
			range_objects
		);
	*/

		// 250 range gets everything in the surrounding area,
		worldobjects.getRange(
			world_view_center.x - 250, world_view_center.y - 250,
			world_view_center.x + 250, world_view_center.y + 250,
			range_objects
		);

		SceneObjects.clearWorldObjects();

		for (RealmWorldObject* obj : range_objects) {
			SceneObjects.addWorldObject(obj);
		}

	}
}

void RealmTerrain::updateScene(int view_level, RealmLocation view_center, RealmSceneView* rsv) {

	updateObjectCollection(view_level, view_center);

	rsv->setSceneObjectCollection(SceneObjects);
}

// maybe?
void RealmTerrain::update() {
}

void RealmTerrain::draw(SDL_Renderer* renderer) {
}

