#ifndef __REALMVIEW_H
	#define __REALMVIEW_H

#include <SDL2/SDL.h>
#include "RealmLocation.h"
#include "XYLocation.h"
#include "RealmSceneView.h"
#include "RealmCoordinateDisplay.h"
#include "RealmTerrain.h"

class HexGridDisplay;

class RealmView {
	private:
		/*
			View Modes:
				1 = Tactical "Scene" Mode, closest in.  1 Hex = 1 Space Tile
					View Range: Current Zone and all Surrounding Zones ( Zone Radius 1, each zone with a radius of 13 space hexes ).
					(Zoom Level 1)

				2 = Explore / Survey "Zone" Mode, second level out.  1 Hex = 1 "Scene".  (1 "Scene" = 7^4 "Spaces", radius 13 hexes)
					View Range: Current Region (Region Radius 0)
					(Zoom Level 4)

				3 = Overview "Region" Mode, third level.  1 Hex = 1 "Zone", (1 "Zone" = 7^4 "Scenes", radius 13 hexes)
					View Range: Current Realm up to its borders, hiding unexplored Regions
					(Zoom Level 7)

					1 Region = 7^4 "Zones"
					1 "Realm" = 7 Regions

				4 = All Realms "World" View  ( 7 Realms ) 1 Big Hex = 1 "Realm" (1 Realm = 7 "Regions")
					View Range: 7 Realms with  7 region hexes within them
					(Zoom Level 10)

		*/
		int ViewMode;
		int TargetViewMode;

		// center of the rendering window
		XYLocation WindowCenter;

		// Base Radius is the radius (in pixels) of a single hex on the currently zoomed level
		int BaseRadius;

		// controls elements related to terrain / map rendering
		RealmTerrain TerrainController;


		RealmSceneView SceneView;
		RealmSceneView ZoneView;
		RealmSceneView RegionView;
		RealmSceneView RealmWorldView;

		// location at the center of focus
		RealmLocation ViewCenter;
		// while moving to a new spot, holds the destination location
		RealmLocation TargetCenter;

		bool AnimatingMove, AnimatingZoom;

		// will this be needed / wanted again?
		// RealmCoordinateDisplay CoordinateMap;

		void updateScene(int view_level);

		RealmSceneView& getRealmSceneView(int view_level);

	public:
		RealmView(XYLocation center, int window_width, int window_height) ;
		void init();
		void cleanup();

		RealmSceneView* getFocusedRealmSceneView();

		int getBaseRadius() { return BaseRadius; }



		RealmSceneView* getZoneView() { return &ZoneView; }

		RealmLocation getCurrentLocation() { return ViewCenter; }

		XYLocation getWindowCenter() { return WindowCenter; }

		void moveFinished(int source_view_mode, HexSpiralCoordinate targetpos);
		void zoomFinished(int source_view_mode);

		double getZoomScale(int level);

		bool isInvertedZoom();

		void autoUpdateObjects();

		// todo move out of TerrainController
		RealmObject* getCursorObject() { return TerrainController.getCursor(); }
		void toggleCursorCamera() { getFocusedRealmSceneView()->toggleCursorCamera(); }
		void cycleSelectedObject();

		void activateMapDisplay();
		void mapPopulateStep();

		void increaseBaseRadius();
		void decreaseBaseRadius();

		// beginMovementKey to be called to start a movement action when a key is pressed and held
		void beginMovementKey(int key_direction);
		// stopMovementKey is called when a key is released
		void stopMovementKey(int key_direction);

		// starts the animation / position change for zooms
		void zoomIn();
		void zoomOut();

		void update();

		void updateTextures(SDL_Renderer* renderer);
		void draw(SDL_Renderer* renderer);
};

#endif // __REALMVIEW_H
