#include "RealmZoomAnimator.h"
#include "const7.h"
#include <SDL2/SDL.h>

RealmZoomAnimator::RealmZoomAnimator(int starting_zoom) {
	CurrentZoom = starting_zoom;
	AnimatingZoom = false;
	ZoomSpeed = 0;
	zoomFinishEvent = nullptr;
	zoomStepEvent = nullptr;
}

void RealmZoomAnimator::startZoom(int target, int max_movement_speed) {
	ZoomTarget = target;
	LastZoomLevel = CurrentZoom;
	AnimatingZoom = 1;
	ZoomSpeed++;
	MaxZoomSpeed = max_movement_speed;
}


/* gets the current scale factor of a given level, given current ZoomLevel and ZoomOffset */
double RealmZoomAnimator::getZoomScale(int level) {
	double r;

	if (abs(CurrentZoom - 1) >= 0.001) {
		if (level < CurrentZoom) {
			r = 1 / (pow(HEX_ZOOM_RATIO, (CurrentZoom - level)));
		} else if (level > CurrentZoom) {
			r = pow(HEX_ZOOM_RATIO, level - CurrentZoom);
		} else {
			r = 1;
		}
	} else {
		if (level == 1)
			r = 1;
		else if (level < 1)
			r = 1 / (pow(HEX_ZOOM_RATIO, (1 - level)));
		else if (level > 1)
			r = pow(HEX_ZOOM_RATIO, level - 1);
	}
	return r;
}


void RealmZoomAnimator::update(int base_radius) {

	if (AnimatingZoom) {

		// smoothing / slowing zoom a bit... could be tweaked if needed, depending on interface experience
		int smooth_slow_factor = 4;
		double zoomfactor = 1.0 / (base_radius * smooth_slow_factor);
		double zoomstep;

		// make zoom steps roughly proportional to xy movements based on the radius size
		zoomstep = ZoomSpeed * zoomfactor;

		// threshold of calling it close enough to be finished
		if (abs(ZoomTarget - CurrentZoom) < 0.001) {
			CurrentZoom = ZoomTarget;
		} else if (CurrentZoom < ZoomTarget) {

			if (ZoomTarget - CurrentZoom < zoomstep)
				zoomstep = ZoomTarget - CurrentZoom;

			CurrentZoom += zoomstep;

		} else if (CurrentZoom > ZoomTarget) {

			if (CurrentZoom - ZoomTarget < zoomstep)
				zoomstep = CurrentZoom - ZoomTarget;

			CurrentZoom -= zoomstep;
		}

		if ( ( abs(ZoomTarget - CurrentZoom) >= 0.5 ) && ZoomSpeed < MaxZoomSpeed) {
			ZoomSpeed++;
		} else if (( abs(ZoomTarget - CurrentZoom) <= 0.25) && ZoomSpeed > 2) {
			ZoomSpeed--;
		}

		if ((ZoomTarget == 1 || ZoomTarget == 2) && LastZoomLevel > 1) {
			ZoomProgress = (abs(LastZoomLevel - CurrentZoom) / abs(LastZoomLevel - ZoomTarget) ) * 10;
			if (zoomStepEvent)
				zoomStepEvent(ZoomProgress);
			//
		}


		if ( abs(ZoomTarget - CurrentZoom) < 0.001 ) {
			AnimatingZoom = false;
			CurrentZoom = ZoomTarget;

			ZoomSpeed = 0;
			if (zoomFinishEvent)
				zoomFinishEvent();

		}
	}
}
