#include "SVG2SDL.h"

#include <iostream>

using namespace std;


void SVG2SDL::load(const std::string& filename) {
	if (! (document = Document::loadFromFile(filename)))  {
		cout << "Error: unable to load SVG file " << filename << endl;
	}
}

SDL_Texture* SVG2SDL::renderToTexture(SDL_Renderer* renderer, int width, int height) {

	SDL_Texture* texture;

	std::uint8_t* data;

	Bitmap sprite = document->renderToBitmap(width, height);
	data = sprite.data();
	int depth = 32; // rgba
	int pitch = 4 * width;

	Uint32 rmask, gmask, bmask, amask;
#if SDL_BYTEORDER == SDL_BIG_ENDIAN
	int shift = 0;
	rmask = 0xff000000 >> shift;
	gmask = 0x00ff0000 >> shift;
	bmask = 0x0000ff00 >> shift;
	amask = 0x000000ff >> shift;
#else // little endian, like x86
	rmask = 0x000000ff;
	gmask = 0x0000ff00;
	bmask = 0x00ff0000;
	amask = 0xff000000;
#endif

	SDL_Surface* sdlsurface;

	sdlsurface = SDL_CreateRGBSurfaceFrom((void*)data, width, height, depth, pitch,
						rmask, gmask, bmask, amask);

	texture = SDL_CreateTextureFromSurface(renderer, sdlsurface);

	SDL_FreeSurface(sdlsurface);
		if (texture == nullptr){
		std::cout << "SDL_CreateTextureFromSurface Error: " << SDL_GetError() << std::endl;
	}
	return texture;
}

void SVG2SDL::drawTexture(SDL_Renderer* renderer, SDL_Texture* texture, XYLocation center) {

	int width, height;
	Uint32 format;
	int access;
	SDL_QueryTexture(texture, &format, &access, &width, &height);

	SDL_Rect dstrect;
	dstrect.x = center.getx() - width / 2;
	dstrect.y = center.gety() - height / 2;
	dstrect.w = width;
	dstrect.h = height;

	/* Draw the render on window */
	int success;
	success = SDL_RenderCopy(renderer, texture, NULL, &dstrect); // Copy the texture into render

	if (success != 0) {
		cout << "SDL RenderCopy error" << endl;
	}
}

void SVG2SDL::cleanupTexture(SDL_Texture* texture) {
	SDL_DestroyTexture(texture);
}
