

#include "SceneMoveAnimator.h"
#include "RealmSceneView.h"

SceneMoveAnimator::SceneMoveAnimator(RealmSceneView* sceneview, HexSpiralCoordinate startCoordinate): CurrentCoordinate(startCoordinate), TargetCoordinate(0)
{
	AnimatingMove = false;
	HasUpdatedFrame = false;
	MovementSpeed = 0;
	XOffset = 0; YOffset = 0;
	moveStartEvent = moveFinishEvent = nullptr;
	SceneView = sceneview;
	GridPosition = startCoordinate.getXY(SceneView->isInverted());

	for (int i = 0; i < 10; i++)
		ActiveMovementKeys[i] = false;
}

XYLocation SceneMoveAnimator::getGridPosition()
{
	return GridPosition;
}

void SceneMoveAnimator::setHexPosition(HexSpiralCoordinate newpos) {
	CurrentCoordinate = newpos;
	GridPosition = newpos.getXY(SceneView->isInverted());
	AnimatingMove = HasUpdatedFrame = false;
	MovementSpeed = 0;
	XOffset = YOffset = 0;
}

XYLocation SceneMoveAnimator::getCurrentOffset()
{
	return XYLocation(XOffset, YOffset);
}

XYLocation SceneMoveAnimator::getScreenLocation() {
	return SceneView->getScreenLocation(getGridPosition()) + getCurrentOffset();
}

double SceneMoveAnimator::getZoomScale() {
	return SceneView->getZoomScale(1);
}

// beginMovementKey to be called to start a movement action when a key is pressed and held
void SceneMoveAnimator::beginMovementKey(int key_direction) {
	ActiveMovementKeys[key_direction] = true;
}

// stopMovementKey is called when a key is released
void SceneMoveAnimator::stopMovementKey(int key_direction) {
	ActiveMovementKeys[key_direction] = false;
}

// todo: cleanup... these functions are a confusing mess
void SceneMoveAnimator::beginMove(HexSpiralCoordinate target) {
	TargetCoordinate = target;

	if (moveStartEvent)
		moveStartEvent(target);
	doMove(target.getXY(SceneView->isInverted()));
}

void SceneMoveAnimator::startMove(int direction) {
	TargetCoordinate = CurrentCoordinate.calcMovement(direction);
	if (moveStartEvent)
		moveStartEvent(TargetCoordinate);
	doMove(TargetCoordinate.getXY(SceneView->isInverted()));
}

void SceneMoveAnimator::updateMovementState() {
	int godir;

	if (AnimatingMove) return;

	if (ActiveMovementKeys[9]) {
		godir = SceneView->isInverted() ? 1 : 0;
		startMove(godir);
	} else if (ActiveMovementKeys[7]) {
		godir = 2;
		startMove(godir);
	} else if (ActiveMovementKeys[1]) {
		godir = SceneView->isInverted() ? 4 : 3;
		startMove(godir);
	} else if (ActiveMovementKeys[3]) {
		godir = 5;
		startMove(godir);
	} else if (ActiveMovementKeys[8]) {
		if (!SceneView->isInverted()) {
			godir = 1;
			startMove(godir);
		}
	} else if (ActiveMovementKeys[2]) {
		if (!SceneView->isInverted()) {
			godir = 4;
			startMove(godir);
		}
	} else if (ActiveMovementKeys[4]) {
		if (SceneView->isInverted()) {
			godir = 3;
			startMove(godir);
		}
	} else if (ActiveMovementKeys[6]) {
		if (SceneView->isInverted()) {
			godir = 0;
			startMove(godir);
		}
	}
}


void SceneMoveAnimator::doMove(XYLocation target)
{
	TargetPosition = target;
	MovementSpeed++;
	AnimatingMove = 1;

	int zoomradius = SceneView->getZoomRadius(1);
	double xscalefactor = zoomradius * SceneView->getXScaleFactor(1);
	double yscalefactor = zoomradius * SceneView->getYScaleFactor(1);

	int oldx = GridPosition.getx() * xscalefactor;
	int oldy = GridPosition.gety() * yscalefactor;

	int curx = oldx + XOffset;
	int cury = oldy + YOffset;

	MoveTargetPixel.setx(TargetPosition.getx() * xscalefactor);
	MoveTargetPixel.sety(TargetPosition.gety() * yscalefactor);

	int xdistance = MoveTargetPixel.getx() - curx;
	int ydistance = MoveTargetPixel.gety() - cury;

	if (abs(xdistance) > 0 || abs(ydistance) > 0) {
		double yxratio;
		double angle = 0;
		if (abs(xdistance) > 0) {
			yxratio = (double) abs(ydistance) / (double) abs(xdistance);
			angle = atan(yxratio);
		} else {
			if (abs(ydistance) > 0)
				angle = atan(1) * 2;
		}

		XMoveRatio = cos(angle) * 3;
		YMoveRatio = sin(angle) * 3;
	}
}

bool SceneMoveAnimator::advanceFrame()
{
	if (AnimatingMove && !HasUpdatedFrame) {
		int zoomradius = SceneView->getZoomRadius(1);
		double xscalefactor = zoomradius * SceneView->getXScaleFactor(1);
		double yscalefactor = zoomradius * SceneView->getYScaleFactor(1);

		int oldx = GridPosition.getx() * xscalefactor;
		int oldy = GridPosition.gety() * yscalefactor;

		int curx = oldx + XOffset;
		int cury = oldy + YOffset;

		int xdistance = MoveTargetPixel.getx() - curx;
		int ydistance = MoveTargetPixel.gety() - cury;

		if (abs(xdistance) > 0 || abs(ydistance) > 0) {

			double xmove = MovementSpeed * XMoveRatio;
			double ymove = MovementSpeed * YMoveRatio;

			if (curx < MoveTargetPixel.getx()) {
				if (curx + xmove > MoveTargetPixel.getx())
					xmove = MoveTargetPixel.getx() - curx;
				XOffset += xmove;
			} else if (curx > MoveTargetPixel.getx()) {
				if (curx - xmove < MoveTargetPixel.getx())
					xmove = curx - MoveTargetPixel.getx();
				XOffset -= xmove;
			}

			if (cury < MoveTargetPixel.gety()) {
				if (cury + ymove > MoveTargetPixel.gety())
					ymove = MoveTargetPixel.gety() - cury;
				YOffset += ymove;
			} else if (cury > MoveTargetPixel.gety()) {
				if (cury - ymove < MoveTargetPixel.gety())
					ymove = cury - MoveTargetPixel.gety();
				YOffset -= ymove;
			}
		}

		if ( (abs(xdistance) > 50 || abs(ydistance) > 50 ) && MovementSpeed < 20) {
			MovementSpeed++;
		} else if ((abs(xdistance) < 30 && abs(ydistance) < 30 ) && MovementSpeed > 2) {
			MovementSpeed--;
		}

		HasUpdatedFrame = true;

		// if we have reached our target, reset variables to stop animating
		if (abs(xdistance) == 0 && abs(ydistance) == 0 ) {
			AnimatingMove = false;

			GridPosition = TargetPosition;
			CurrentCoordinate = TargetCoordinate;
			MovementSpeed = 0;
			XOffset = 0;
			YOffset = 0;
			return true;
		}
	}

	return false;
}

void SceneMoveAnimator::update() {
	updateMovementState();
	if (advanceFrame() && moveFinishEvent)
		moveFinishEvent(TargetCoordinate);
}

void SceneMoveAnimator::resetFrameUpdate() {
	HasUpdatedFrame = false;
}
