
#ifndef __SCENEMOVEANIMATOR_H
	#define __SCENEMOVEANIMATOR_H

#include "XYLocation.h"
#include "HexSpiralCoordinate.h"

#include <functional>

class RealmSceneView;

class SceneMoveAnimator {
private:

	RealmSceneView* SceneView;

	bool AnimatingMove;
	int MovementSpeed;
	int MaxMovementSpeed;

	HexSpiralCoordinate CurrentCoordinate;
	HexSpiralCoordinate TargetCoordinate;

	XYLocation GridPosition;
	XYLocation TargetPosition;
	XYLocation MoveTargetPixel;
	double XMoveRatio, YMoveRatio;
	double XOffset, YOffset;
	bool HasUpdatedFrame;


	// true when key is pressed
	bool ActiveMovementKeys[10];

	std::function<void(HexSpiralCoordinate dest)> moveFinishEvent;
	std::function<void(HexSpiralCoordinate dest)> moveStartEvent;

	// in progress -- being moved from RealmView/RealmSceneView
	void startMove(int direction);

	void doMove(XYLocation target);

public:
	SceneMoveAnimator(RealmSceneView* sceneview, HexSpiralCoordinate startCoordinate);

	// beginMovementKey to be called to start a movement action when a key is pressed and held
	void beginMovementKey(int key_direction);
	// stopMovementKey is called when a key is released
	void stopMovementKey(int key_direction);

	void beginMove(HexSpiralCoordinate target);

	void setMoveStartEvent(std::function<void(HexSpiralCoordinate dest)> e) { moveStartEvent = e; }
	void setMoveFinishEvent(std::function<void(HexSpiralCoordinate dest)> e) { moveFinishEvent = e; }

	void updateMovementState();

	void setHexPosition(HexSpiralCoordinate newpos);
	HexSpiralCoordinate getHexPosition()  { return CurrentCoordinate; }

	XYLocation getScreenLocation();
	double getZoomScale();

	XYLocation getGridPosition();
	XYLocation getCurrentOffset();

	void update();

	bool advanceFrame(); // returns true when finished
	void resetFrameUpdate();
};


#endif // __SCENEMOVEANIMATOR_H
