#include "SceneObjectCollection.h"
#include "RealmSceneView.h"
#include "RealmSceneObject.h"
#include "RealmWorldObject.h"
#include "SceneMoveAnimator.h"
#include "RealmSceneObject.h"


SceneObjectCollection::SceneObjectCollection() {
	object_count = 0;
	realm_scene_view = nullptr;
}

void SceneObjectCollection::addObject(RealmSceneObject* object) {
	objects.push_front(object);
	object_count++;
}

void SceneObjectCollection::addWorldObject(RealmWorldObject* object) {
	world_objects.push_front(object);
}

void SceneObjectCollection::clearWorldObjects() {
	world_objects.clear();
}

void SceneObjectCollection::updateRealmSceneView(RealmSceneView* rsv)
{
	SceneMoveAnimator* object_animator;
	if (realm_scene_view != rsv) {
		realm_scene_view = rsv;
		for (RealmSceneObject* &o : objects) {
			object_animator = new SceneMoveAnimator(rsv, o->getSceneCoordinate() );
			o->setObjectAnimator(object_animator);
		}
		for (RealmWorldObject* &o : world_objects) {
			object_animator = new SceneMoveAnimator(rsv, HexSpiralCoordinate(0) );
			o->setObjectAnimator(object_animator);

			o->setRealmSceneView(rsv);

		}
	}
}

void SceneObjectCollection::updateObjects() {
	for (RealmSceneObject* &o : objects) {
		o->update();
	}
	for (RealmWorldObject* &o : world_objects) {
		o->update();
	}
}

void SceneObjectCollection::resetUpdateFlags() {
	for (RealmSceneObject* &o : objects) {
		o->resetUpdateFlags();
	}
	for (RealmWorldObject* &o : world_objects) {
		o->resetUpdateFlags();
	}
}

void SceneObjectCollection::drawObjects(SDL_Renderer* renderer) {
	for (RealmSceneObject* &o : objects) {
		o->draw(renderer);
	}
	for (RealmWorldObject* &o : world_objects) {
		o->draw(renderer);
	}
}

